
clear
load modtran.atmosphere.mid_summer.dat 
data = modtran ;
clear modtran

alt = data(:,1) ;
press = data(:,2) ;
temp = data(:,3) ;
h2o_number = data(:,4) ;
o3_number = data(:,6) ;
ch4_number = data(:,8) ;
n2o_number = data(:,6) ;

clear data


edge_pressure = [976.322 956.654 903.800 853.500 805.700 715.900 633.800 492.143 377.200 244.787 129.667  68.00] ;
edge_temp     = [302.14  300.50  295.85  297.05  292.95  284.16  274.35  264.75  248.75 225.15 203.25 224.92] ;
delta_alt = [0.18 0.5 0.5 0.5 1.0 1.0 2.0 2.0 3.0 4.0 53.0] ;

%interpolating number concentration in cm^-3
h2o_concent = h2o_number ./ (100000 .* delta_alt') ;
o3_concent = o3_number ./ (100000 .* delta_alt') ;

%interpolating the concentration
log_press = log10(press) ;
log_press_edge = log10(edge_pressure) ;

for i = 2:length(edge_pressure) - 1 ;
  frctn = (log_press_edge(i) - log_press(i-1)) / (log_press(i) - log_press(i-1)) ;
  edge_h2o_concent(i) = frctn * (h2o_concent(i) - h2o_concent(i-1)) + h2o_concent(i-1) ;
  edge_o3_concent(i) = frctn * (o3_concent(i) - o3_concent(i-1)) + o3_concent(i-1);
end

frctn = (log_press(1) - log_press_edge(1)) / (log_press_edge(2) - log_press_edge(1)) ;
edge_h2o_concent(1) =  (h2o_concent(1) - (frctn * edge_h2o_concent(2))) / (1.0 - frctn) ;
edge_o3_concent(1) =  (o3_concent(1) - (frctn * edge_o3_concent(2))) / (1.0 - frctn) ;
frctn = (log_press(11) - log_press_edge(11)) / (log_press_edge(12) - log_press_edge(11)) ;
edge_h2o_concent(length(edge_pressure)) = (h2o_concent(11) - (1.0 - frctn) *  edge_h2o_concent(11)) / frctn ;
edge_o3_concent(length(edge_pressure)) = (o3_concent(11) - (1.0 - frctn) *  edge_o3_concent(11)) / frctn ;



% Avogadro constant mol^-1
abg = 6.02e23 ;
% Boltzmann constant, J K^-1
bolz = 1.38e-23 ;
% gravity
g = 9.81 ;
% air mass g mol^-1
air = 28.98 ;
%Loschmidt's number
losh = 2.687e19 ;

% h2o mass g mol^-1
h2o = 18.016 ;

% ozone mass g mol^-1
o3 = 48.0 ;

% n2o mass in g mol^-1
n2o = 44.0128

% methane mass in g mol^-1
ch4 = 16.0426

% concentration of air in cm^-3
edge_concent_air = (edge_pressure .* 100.0) ./ (bolz .* edge_temp) ;
edge_concent_air = edge_concent_air .* 1.0e-6 ;

% mixing ratio
mix_h2o = (edge_h2o_concent .* h2o) ./ (edge_concent_air .* air) ;
mix_h2o = mix_h2o .* 1000.0 ;

mixo3 = (edge_o3_concent .* o3) ./ (edge_concent_air .* air) ;

for i = 1:length(edge_pressure) - 1
  mass_air(i) = 100.0 * (edge_pressure(i) - edge_pressure(i+1)) / g ; % in kg m^-2
  frctn = (log_press(i) - log_press_edge(i)) / (log_press_edge(i+1) - log_press_edge(i)) ;
  ave_mix_h2o(i) = frctn * (mix_h2o(i+1) - mix_h2o(i)) + mix_h2o(i) ;
  mass_h2o(i) = mass_air(i) * ave_mix_h2o(i) ./  10000.0; % in g cm^-2

  ave_mix_o3(i) = frctn * (mixo3(i+1) - mixo3(i)) + mixo3(i) ;
  mass_o3(i) = 1000.0 * mass_air(i) * ave_mix_o3(i) ./  10000.0; % in g cm^-2
end

 h2o_number_kato = abg .* mass_h2o ./ h2o
 o3_number_kato = abg .* mass_o3 ./ o3



