
clear
load modtran.atmosphere.mid_summer.dat 
data = modtran ;
clear modtran

alt = data(:,1) ;
press = data(:,2) ;
temp = data(:,3) ;
h2o_cm = data(:,4) ;
o3_cm = data(:,5) ;
ch4_cm = data(:,8) ;
n2o_cm = data(:,6) ;

clear data


% Avogadro constant mol^-1
abg = 6.02e23 ;
% Boltzmann constant, J K^-1
bolz = 1.38e-23 ;
% gravity
g = 9.81 ;
% air mass g mol^-1
air = 28.98 ;
%Loschmidt's number
losh = 2.687e19 ;

% h2o mass g mol^-1
h2o = 18.016 ;

% ozone mass g mol^-1
o3 = 48.0 ;

% n2o mass in g mol^-1
n2o = 44.0128

% methane mass in g mol^-1
ch4 = 16.0426

%number of air molecule in 1 km thick layer
% n_air is in # m^-2/km
n_air = (press .* 100.0 .* 1000.0) ./ (bolz .* temp) ; 

% mol of air in the column
% mol_air is in mol m^-3
mol_air = n_air ./ abg ;

% mass of air in gram
% mass of air is in g m^-3
mass_air = mol_air .* air ;

% mass of h2o
% mass_h2o is in g cm^-2/km
mass_h2o = (h2o_cm .* losh .* h2o) ./ abg ;

% mass of o3
% mass_o3 is in g cm^-2/km
mass_o3 = (o3_cm .* losh .* o3) ./ abg ;

% number of n2o in #cm^-2 km^-1
n_n2o = n2o_cm .* losh ;

% number of ch4 in #cm^-2 km^-1
n_ch4 = ch4_cm .* losh ;


% mixing ratio in g / kg
mix_h2o = 1.0e7 .* mass_h2o ./ mass_air

% mixing ratio in g / g
mix_o3 = 1.0e4 .* mass_o3 ./ mass_air

% mixing ration in # / # 
mix_n2o = 1.0e4 * 1.0e6 .* n_n2o ./ n_air
mix_ch4 = 1.0e4 * 1.0e6 .* n_ch4 ./ n_air
