      SUBROUTINE EPHEMS (LAT,LONG,DAY,MONTH,YEAR,TIME,P5,P6,P7,P8,
     *  A,R,Z,HA,DEC)
C
C       SUBROUTINE TO COMPUTE SOLAR POSITION
C
C      WRITTEN BY WAYNE H WILSON, JR.
C      VISIBILITY LABORATORY
C      SCRIPPS INSTITUTION OF OCEANOGRAPHY
C      UNIVERSITY OF CALIFORNIA, SAN DIEGO
C      LA JOLLA, CALIFORNIA, 92093
C      PH. (714)-294-5534
C      25 FEB 1980
C
      IMPLICIT REAL*8(P)
      IMPLICIT INTEGER*4(I-N)
      REAL*4 LAT,LONG,TIME
      REAL*8 DARSIN,DARCOS,DTAN
      REAL*4 A, R, Z, HA, DEC
      REAL*8 DEGRD,PI,TOHMS,ONE
      INTEGER*4 P5,P6,P7,P8,P10
      INTEGER*4 DAY,MONTH,YEAR
C
C     LAT - LATITUDE (IN DEGREES AND FRACTION OF DEGREES), P19
C     LONG - LONGITUDE (IN DEGREES AND FRACTIONS OF DEGREES),P20
C     DAY - DAY OF THE MONTH,P16
C     MONTH - MONTH OF YEAR,P17
C     YEAR - YEAR,P18
C     TIME - TIME (HHMM.SS) LOCAL STANDARD LST OR GMT, P4
C     P5 - TIME TYPE (0 - LST, 1 - GMT)
C     P6 - APPARENT NOON CALCULATIONS (1-YES, 0-NO)
C     P7 - PRINT (0-NONE, 1-DATE, 2-DATA, 3 - NA COMPAR)
C     P8 - APPROXIMATION (0-FULL, 1-NO PLANETS, 2-NO LUNAR)
C
C      RETURNED VARIABLES
C
C      A - AZIMUTH ANGLE(IN DEGREES AND FRACTION OF DEGREE)
C     R - RADIUS VECTOR
C     Z - ZENITH ANGLE(IN DEGREES AND FRACTION OF DEGREE)
C     TIME - APPARENT NOON IF P6 = 1 (LST OR GMT SEE P5)
C
C     P21 - TIME (IN FRACTIONS OF DAY)
C     P51 - GMT IN DEGREES
C     P53 - ZONE LONGITUDE
C
C
        DATA ONE/1.0D0/
C
C
      DTAN(P)=DSIN(P)/DCOS(P)
      DARSIN(P)=DATAN2(P,DSQRT(1.0D0-P**2))
      DARCOS(P) = PID2 - DARSIN(P)
C
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C
C
        PID2 = DATAN(1.0D0)*2.0D0
        PI = PID2*2.0D0
      DEGRD = PI/180D0
C
C++++++++++++++++++++++++++++++++++++++++++++
C      P57 - (ET-UT), DIFFERENCE OF EPHEMERIS & UNIVERSAL TIME
      P57 = 0
C
C
      P16 = DAY
      P17 = MONTH
      P18 = YEAR
      P19 = LAT
      P20 = LONG
      P54 = 10
      P55 = 14
      P10 = P7
    1 CONTINUE
      P53 = 15.*AINT((7.5+ABS(LONG))/15.0)*SIGN(1.0,LONG)
      P11 = P53
      IF(P5.EQ.1) P11=0D0
      IF(P6.NE.1) GO TO 10
      P56 = (P54+P55)/2D0
      P51 = P56/24D0*360D0+P53
      GO TO 20
   10 CONTINUE
      P21 = (AMOD(TIME,1.0)*100/3600+AMOD(AINT(TIME)/100.,1.0)*100./60.
     *    + AINT(AINT(TIME)/100.0))/24.0
      P51 = P21*360D0+P11
   20 CONTINUE
      I = P18
      J = P17+1
      IF(J.GT.3) GO TO 40
      J = J + 12
      I = I - 1
   40 CONTINUE
      P13 = INT(I*365.25) + INT(J*30.6) + P16
      P22 = P13 - 694038.5D0
      P11 = P18 + P17/100.D0 + P16/1.0D4
      IF(P11.LT.1900.0228D0) P22 = P22 + 1
      IF(P11.LT.1800.0228D0) P22 = P22+1
      P23 = (P51/360D0+P22+P57)/36525D0
      P22 = P23*36525D0
C
C      MEAN LONGITUDE - P24
C
      P11 = 279.69668D0
      P12 = 0.9856473354D0
      P13 = 3.03D-4
      P24 = P11 + DMOD(P12*P22,360D0) + P13*P23**2
      P24 = DMOD(P24,360D0)
C
C     MEAN ANOMALY - P25
C
      P11 = 358.47583D0
      P12 = 0.985600267D0
      P13 = -1.5D-4
      P14 = -3.D-6
      P25 = P11 + DMOD(P12*P22,360D0) + P13*P23**2 + P14*P23**3
      P25 = P11 + DMOD(P12*P22,360D0) + P13*P23**2 + P14*P23**3
      P25 = DMOD(P25,360D0)
C
C     ECCENTRICITY - P26
C
      P11 = 0.01675104D0
      P12 = -4.18D-5
      P13 = -1.26D-7
      P26 = P11 + P12*P23 + P13*P23**2
      P11 = P25*DEGRD
      P12 = P11
C
C     ECCENTRIC ANOMALY - P13 (TEMP)
C
  100 CONTINUE
      P13 = P12
      P12 = P11 + P26*DSIN(P13)
      IF(DABS((P12-P13)/P12).GT.1.0D-8) GO TO 100
      P13 = P12/DEGRD
C
C     TRUE ANOMALY - P27
C
      P27 =2.0D0*DATAN(DSQRT((1.0D0+P26)/(1.0D0-P26))*DTAN(P13/2.0D0
     *   *DEGRD))/DEGRD
      IF(DSIGN(1.0D0,P27).NE.DSIGN(1.0D0,DSIN(P13*DEGRD))) P27 = P27 + 1
      IF(DSIGN(1.0D0,P27).NE.DSIGN(1.0D0,DSIN(P13*DEGRD)))
     *  P27 = P27 + 180.0D0
      IF(P27.LT. 0.0D0) P27 = P27 + 360D0
C
C     RADIUS VECTOR - R
C
      R = 1.0D0 - P26*DCOS(P13*DEGRD)
C
C     ABERRATION - P29
C
      P29 = -20.47/R/3600.0
C
C      MEAN OBLIQUITY - P43
C
      P11 = 23.452294D0
      P12 = -0.0130125D0
      P13 = -1.64D-6
      P14 = 5.03D-7
      P43 = P11 + P12*P23 + P13*P23**2 + P14*P23**3
C
C     MEAN ASCENSION - P45
C
      P11 = 279.6909832D0
      P12 = 0.98564734D0
      P13 = 3.8708D-4
      P45 = P11 + DMOD(P12*P22,360D0) + P13*P23**2
      P45 = DMOD(P45,360D0)
C
C-----------------------------------------------------------------------
C
CAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
C
C     NUTATION AND LONGITUDE PERT
C
      IF(P8.GT.1) GO TO 300
C-----------------------------------------------------------------------
C
C      MOON'S MEAN ANOMALY - P28
      P11 = 296.104608D0
      P12 = 1325D0*360D0
      P13 = 198.8491083D0
      P14 = .00919167D0
      P15 = 1.4388D-5
      P28 = P11 + DMOD(P12*P23,360D0) + P13*P23 + P14*P23**2 +
     *  P15*P23**3
      P28 = DMOD(P28,360D0)
C
C     MEAN ELONGATION OF MOON - P30:
C
      P11 = 350.737486
      P12 = 1236D0 * 360D0
      P13 = 307.1142167D0
      P14 = -1.436D-3
      P30 = P11 + DMOD(P12*P23,360D0) + P13*P23 + P14*P23**2
      P30 = DMOD(P30,360D0)
C
C     MOON LONG OF ASCENDING NODE - P31:
C
      P11 = 259.183275D0
      P12 = -5D0 * 360D0
      P13 = -134.142008D0
      P14 = 2.0778D-3
      P31 = P11 + DMOD(P12*P23,360D0) + P13*P23 + P14*P23**2
      P31 = DMOD(P31,360D0)
C
C     MEAN LONG OF MOON - P32:
C
      P11 = 270.434164D0
      P12 = 1336D0 * 360D0
      P13 = 307.8831417D0
      P14 = -1.1333D-3
      P32 = P11 + DMOD(P12*P23,360D0) + P13*P23 + P14*P23**2
      P32 = DMOD(P32,360D0)
C
C     MOON PERTURBATION OF SUN LONG - P74:
C
      P74 = 6.454D0*DSIN(P30*DEGRD) + .013D0*DSIN(3D0*P30*DEGRD) +
     *  .177D0*DSIN((P30+P28)*DEGRD) - .424D0*DSIN((P30-P28)*DEGRD)
       P74 = P74 + 0.039D0*DSIN((3D0*P30-P28)*DEGRD)
     * - 0.064D0*DSIN((P30+P25)*DEGRD) + 0.172D0*DSIN((P30-P25)*DEGRD)
      P74 = P74/3600D0
        P33 = P74
C
C     NUTATION OF LONG - P34
C
      P34 = -(17.2327D0-.017D0*P23) * DSIN(P31*DEGRD) +
     *  .2088D0*DSIN(2D0*P31*DEGRD) - .2037D0*DSIN(2D0*P32*DEGRD)
      P34 = P34 - 1.2729D0*DSIN(2D0*P24*DEGRD) + .1261D0*DSIN(P28*DEGRD)
      P34 = P34/3600D0
C
C     NUTATION IN OBLIQUITY - P34:
C
      P35 = 9.210D0*DCOS(P31*DEGRD) + .5522D0*DCOS(2D0*P24*DEGRD) -
     *  .09D0*DCOS(2D0*P31*DEGRD) + .0889D0*DCOS(2D0*P32*DEGRD)
      P35 = P35/3600D0
C
C     INEQUALITIES OF LONG PERIOD - P36:
C
      P36 = .266D0*DSIN((31.8D0+119D0*P23)*DEGRD) +
     *((1.882D0-.016D0*P23)*DEGRD) * DSIN((57.24D0+150.27D0*P23)*DEGRD)
      P36 = P36 + .202D0*DSIN((315.6D0+893.3D0*P23)*DEGRD) +
     * 1.089D0*P23**2 + 6.4D0*DSIN((231.19D0+20.2D0*P23)*DEGRD)
      P36 = P36/3600D0
C
C     MOON MEAN ARGUMENT OF LATITUDE - P63:
C
      P11 = 11.250889D0
      P12 = 1342D0*360D0
      P13 = 82.02515D0
      P14 = .003211D0
      P63 = P11 + DMOD(P12*P23,360D0) + P13*P23 + P14*P23**2
C
C-----------------------------------------------------------------------
C
CBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB
C
C     PERTURBATIONS DUE TO PLANETS - P33:
C
      IF(P8.GT.0) GO TO 300
C
C     VENUS:
C
C     MEAN ANOMALY OF VENUS - P37:
C
      P11 = 212.603222D0
      P12 = 162D0*360D0
      P13 = 197.803875D0
      P14 = 1.286D-3
      P37 = P11 + DMOD(P12*P23,360D0) + P13*P23 + P14*P23**2
      P37 = DMOD(P37,360D0)
      P11 = 4.838D0*DCOS((299.171D0+P37-P25)*DEGRD) +
     *  5.526D0*DCOS((148.5259D0+2D0*P37-2D0*P25)*DEGRD)
      P11 = P11 + 2.497D0*DCOS((316.5759D0+2D0*P37-3D0*P25)*DEGRD) +
     *   .666D0*DCOS((177.71D0+3D0*P37-3D0*P25)*DEGRD)
      P11 = P11 + 1.559D0*DCOS((345.4559D0+3D0*P37-4D0*P25)*DEGRD) +
     * 1.024D0*DCOS((318.15D0+3D0*P37-5D0*P25)*DEGRD)
      P70 = P11
      P33 = P33+P11/3600D0
C
C     PERT. OF LATITUDE OF SUN BY VENUS - P61:
C
      P61 = .21D0*DCOS((151.8D0+3D0*P37-4D0*P25)*DEGRD) +
     * .092D0*DCOS((93.7D0+P37-2D0*P25)*DEGRD) +
     * .067D0*DCOS((123D0+2D0*P37-3D0*P25)*DEGRD)
C
C     MARS:
C
C     MEAN ANOMALY OF MARS - P38:
C
      P11 = 319.529022D0
      P12 = 53D0*360D0
      P13 = 59.8592194D0
      P14 = 1.8083D-4
      P38 = P11 + DMOD(P12*P23,360D0) + P13*P23 + P14*P23**2
      P38 = DMOD(P38,360D0)
      P11 = .273D0*DCOS((217.7D0-P38+P25)*DEGRD) +
     *  2.043D0*DCOS((343.888D0-2D0*P38+2D0*P25)*DEGRD)
      P11 = P11 + 1.77D0*DCOS((200.4017D0-2D0*P38+P25)*DEGRD)+
     * .425D0*DCOS((338.88D0-3D0*P38+2D0*P25)*DEGRD)
      P11 = P11 + .5D0*DCOS((105.18D0-4D0*P38+3D0*P25)*DEGRD) +
     *  .585D0*DCOS((334.06D0-4D0*P38+2D0*P25)*DEGRD)
      P71=P11
      P33 = P33+P11/3600D0
C
C     JUPITER:
C
C     MEAN ANOMALY OF JUPITER - P39:
C
      P11 = 225.3225D0
      P12 = 8D0*360D0
      P13 = 154.583D0
      P39 = P11 + DMOD(P12*P23,360D0) + P13*P23
      P39 = DMOD(P39,360D0)
      P11 = 7.208D0*DCOS((179.5317D0-P39+P25)*DEGRD) +
     *  2.6D0*DCOS((263.2167D0-P39)*DEGRD)
      P11 = P11 + 2.731*DCOS((87.1450D0-2D0*P39+2D0*P25)*DEGRD)
      P11 = P11 + 1.61D0*DCOS((109.4933D0-2D0*P39+P25)*DEGRD) +
     *  .556D0*DCOS((82.65D0-3D0*P39+2D0*P25)*DEGRD)
      P72=P11
      P33 = P33+P11/3600D0
C
C     PERT OF SOLAR LATITUDE BY JUPITER - P62:
C
      P62 = .166D0*DCOS((265.5D0-2D0*P39+P25)*DEGRD)
C
C     SATURN:
C
C     MEAN ANOMALY OF SATURN - P40:
C
      P11 = 175.613D0
      P12 = 3D0*360D0
      P13 = 141.794D0
      P40 = P11 + DMOD(P12*P23,360D0) + P13*P23
      P40 = DMOD(P40,360D0)
      P11 = .419*DCOS((100.58D0-P40+P25)*DEGRD) +
     *  .32D0*DCOS((269.46D0-P40)*DEGRD)
      P73=P11
      P33 = P33+P11/3600D0
C
C-----------------------------------------------------------------------
C
CBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB
C
CAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
C
C     PARAMETERS
C
  300 CONTINUE
C
C     PRECESSION:
C
      P42 = (50.2564D0+.0222D0*(P18-1900D0)/100D0) *
     *  (P23-(P18-1900D0)/100D0) * 100D0/3600D0
C
C     APPARENT LONGITUDE - P41:
C
      P41 = P27-P25+P24+P29+P33+P36+P34
C
C     SOLAR LATITUDE - P60:
C
      P60 = (.576D0*DSIN(P63*DEGRD) + P61+P62)/3600D0
C
C      OBLIQUITY - P75
C
      P75 = P35 + P43
C
C     APPARENT RIGHT ASCENSION - P44:
C
      P44  =  DATAN(DTAN(P41*DEGRD)*DCOS(P75*DEGRD))/DEGRD
      IF(DSIGN(ONE,P44).NE.DSIGN(ONE,DSIN(P41*DEGRD))) P44 = P44+180D0
      IF(P44.LT.0.D0) P44 = 360D0+P44
C
C     EQUATION OF TIME - P46:
C
      P46 = P45-P44
      IF(P46.GT.180D0) P46 = P46-360D0
C
C     HOUR ANGLE - P48:
C
      P48 = P51+P46-180D0
      HA = P48
C
C     LOCAL HOUR ANGLE - P49
C
      P49  =  P48-P20
      P49 = DMOD(P49,360D0)
      IF(P49.LT.0) P49 = P49 + 360D0
C
C
C
C     DECLINATION - P47:
C
      P47 = DARSIN(DSIN(P41*DEGRD)*DSIN(P75*DEGRD)*DCOS(P60*DEGRD) +
     * DSIN(P60*DEGRD) * DCOS(P75*DEGRD))/DEGRD
      DEC = P47
C
C     ZENITH ANGLE - Z:
C
      Z = DARCOS(DSIN(P19*DEGRD)*DSIN(P47*DEGRD) + DCOS(P19*DEGRD) *
     *  DCOS(P47*DEGRD) * DCOS(P49*DEGRD))/DEGRD
C
C     AZIMUTH ANGLE - A:
C
      P11 = (-DSIN(P19*DEGRD) * DCOS(P49*DEGRD) * DCOS(P47*DEGRD) +
     *  DSIN(P47*DEGRD) *DCOS(P19*DEGRD))/DSIN(Z*DEGRD)
      IF(DABS(P11).GT.1.0D0) P11 = DSIGN(1.0D0,P11)
      A= DARCOS(P11)/DEGRD
      IF(DSIGN(1.D0,-DCOS(P47*DEGRD)  * DSIN(P49*DEGRD)/DSIN(Z*DEGRD))
     * .NE.DSIGN(1D0,DSIN(A*DEGRD)))  A = 360-A
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      IF(P10.EQ.0) GO TO 30
      CALL NUPAGE
      PRINT 2000,MONTH,DAY,YEAR
2000  FORMAT(' MONTH',I5,/,' DAY',I5,/,' YEAR',I6)
      IF(P8.EQ.0) PRINT 2001
      IF(P8.EQ.1) PRINT 2002
      IF(P8.EQ.2) PRINT 2003
2001  FORMAT(' FULL COMPUTATIONS')
2002  FORMAT(' NO PLANETS')
2003  FORMAT(' NO MOON')
      PRINT 2004,LAT,LONG
 2004  FORMAT(' LATITUDE ', F10.4,/,' LONGITUDE',F10.4)
        IF(P6.NE.1.AND.P5.EQ.0) PRINT 2050,TIME
 2050  FORMAT(' TIME (LST)',F10.2)
        IF(P6.NE.1.AND.P5.EQ.1) PRINT 2051,TIME
 2051  FORMAT(' TIME (GMT)',F10.2)
C
        IF(P10.LT.2) GO TO 30
C
CDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
C
      PRINT 2005,P23,P22
2005  FORMAT(' T',F15.6,' D',F15.6)
      P0 = TOHMS(P24)
      PRINT 2006,P0
2006  FORMAT(' MEAN LONG',F15.5)
      P0 = TOHMS(P25)
      PRINT 2007,P0
2007  FORMAT(' MEAN ANOMALY',F15.5)
      PRINT 2008,P26
2008  FORMAT(' ECCENTRICITY',F15.5)
      P0 = TOHMS(P27-P25)
      PRINT 2009,P0
2009  FORMAT(' EQUATION OF CENTER',F15.5)
      PRINT 2010,R
2010  FORMAT(' RADIUS VECTOR',F15.5)
      P0 = TOHMS(P29)
      PRINT 2011,P0
 2011 FORMAT (' ABERRATION ',F15.5)
      P0 = TOHMS(P43)
      PRINT 2012,P0
2012  FORMAT(' MEAN OB',F15.5)
      P0 =  TOHMS(P45/360D0*24D0)
      PRINT 2013,P0
2013  FORMAT(' MEAN ASCENSION',F15.5)
        IF(P10.GT.1) GO TO 35
      P0 = TOHMS(P74)
      PRINT 2014, P0
 2014 FORMAT (' P MOON',F15.5)
        IF(P10.GT.0) GO TO 35
      P0 = P70/3600D0
      P0 = TOHMS(P0)
      PRINT 2015, P0
 2015 FORMAT (' P VENUS',F15.5)
      P0 = P61/3600D0
      P0 = TOHMS(P0)
      PRINT 2016, P0
 2016 FORMAT (' P LAT BY VENUS ',F15.5)
      P0 = P71/3600D0
      P0 = TOHMS(P0)
      PRINT 2017, P0
 2017 FORMAT (' P MARS ',F15.5)
      P0 = P72/3600D0
      P0 = TOHMS(P0)
      PRINT 2018, P0
 2018 FORMAT (' P JUPITER ',F15.5)
      P0 = P62/3600D0
      P0 = TOHMS(P0)
      PRINT 2019, P0
 2019 FORMAT (' P OF LAT BY JUP ',F15.5)
      P0 = P73/3600D0
      P0 = TOHMS(P0)
      PRINT 2020, P0
 2020 FORMAT (' P SATURN ',F15.5)
C
   35  CONTINUE
C
      P0 = TOHMS(P33)
      PRINT 2021, P0
 2021 FORMAT (' PERTURBATIONS ',F15.5)
      P0 = TOHMS(P36)
      PRINT 2022, P0
 2022 FORMAT (' LONG PERIOD ', F15.5)
      P0 = TOHMS(P34)
      PRINT 2023, P0
 2023 FORMAT (' NUTATION OF LONG ', F15.5)
      P0 = TOHMS(P35)
      PRINT 2024, P0
 2024 FORMAT (' NUT OBLIQUITY ', F15.5)
      P0 = TOHMS(P42)
      PRINT 2025, P0
 2025 FORMAT (' PRECESSION ', F15.5)
      P0 = TOHMS(P41)
      PRINT 2026, P0
 2026 FORMAT (' APPAR. LONG. ',F15.5)
      P0 = TOHMS(P60)
      PRINT 2027, P0
 2027 FORMAT (' SOLAR LATITUDE ',F15.5)
      P0 = TOHMS(P75)
      PRINT 2028, P0
 2028 FORMAT (' OBLIQUITY ',F15.5)
      P0 = P44/360D0*24D0
      P0 = TOHMS(P0)
      PRINT 2029, P0
 2029 FORMAT (' APPAR. ASCENSION ',F15.5)
      P0 = P46/360D0*24D0
      P0 = TOHMS(P0)
      PRINT 2030, P0
 2030 FORMAT (' EQUATION OF TIME ', F15.5)
      P0 = TOHMS(P49)
      PRINT 2032, P0
 2032 FORMAT (' LHA = ',F15.5)
      P0 = TOHMS(P47)
      PRINT 2033, P0
 2033 FORMAT (' DECLIN. ',F15.5)
C
CDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
   30 CONTINUE
C
CEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
C
C
      IF(P6.EQ.0) GO TO 500
      P12 = P53/360D0*24D0
      IF(P5.EQ.0) P12=0D0
      P10 = 0
      IF(AMOD(A,180.).GT.0.01) GO TO 400
      P11 = P56+P12
      IF(P11.LT.0D0) P11 = P11+24D0
      P13 = DMOD(P11,1D0)*60.0D0
      TIME = DINT(P11)*100.0D0+DINT(P13)
     * + DMOD(P13,1.D0)*60.0D0/100.D0
      IF(P56+P12.LT.0D0) TIME = -TIME
      RETURN
C
C
  400 CONTINUE
      IF(A.LT.90.0.OR.A.GT.270.0) GO TO 410
      IF(A.LE.180.) GO TO 405
  402  CONTINUE
      P55 = P56
      GO TO 1
  405  CONTINUE
        P54 = P56
        GO TO 1
  410 CONTINUE
      IF(A.LT.90.0) GO TO 405
        GO TO 402
C
C
  500 CONTINUE
C
CEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
C
        IF(P10.EQ.0) GO TO 550
       P0 = A
      P0 = TOHMS(P0)
      PRINT 2034, P0
 2034 FORMAT (' A      DDD.MMSSS ',F15.5)
      P0 = Z
      P0 = TOHMS(P0)
      PRINT 2035, P0
 2035 FORMAT (' Z       DD.MMSSS ',F15.5)
C
  550  CONTINUE
      IF(P7.LT.3) RETURN
C
CFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C
      IF(P10.EQ.0) GO TO 600
      P52 = P46
      P0 = P41-(P29+P34)
      P0 = TOHMS(P0)
      PRINT 2036, P0
 2036 FORMAT (' LONGITUDE ',F15.6)
      P0 = (P29+P34)*10000
      P0 = TOHMS(P0)
      PRINT 2037, P0
 2037 FORMAT (' REDN TO LONG ',F15.6)
      P0 = P60*10000
      P0 = TOHMS(P0)
      PRINT 2038, P0
 2038 FORMAT (' LATITUDE ',F15.6)
      P0 = P42*10000
      P0 = TOHMS(P0)
      PRINT 2039, P0
 2039 FORMAT (' PRECESSION ',F15.6)
      P0 = P34*10000
      P0 = TOHMS(P0)
      PRINT 2040, P0
 2040 FORMAT (' NUT IN LONG ',F15.6)
      P0 = P35*10000
      P0 = TOHMS(P0)
      PRINT 2041, P0
 2041 FORMAT (' NUT IN OBLI ',F15.6)
      P0 = TOHMS(P43)
      PRINT 2042, P0
 2042 FORMAT (' OBLIQUITY ',F15.6)
      P0 = P44/360D0*24D0
      P0 = TOHMS(P0)
      PRINT 2043, P0
 2043 FORMAT (' APP RT. ASCEN ',F15.6)
      P0 = TOHMS(P47)
      PRINT 2044, P0
 2044 FORMAT (' APP DECLINTION ',F15.6)
C
  600 CONTINUE
      P16 = P16+1.D0
      IF(P10.EQ.0) GO TO 610
      P10 = 0D0
      GO TO 1
  610 CONTINUE
      P11 = 180.D0-P52-(P46-P52)*((-P52+180.D0)/360D0)
      P0 = TOHMS(P11/360.D0*24D0)
      PRINT 2045,P0
 2045 FORMAT (' ET ',F15.6)
C
C
      RETURN
C
CFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C
C              *** END SUBROUTINE EPHEMS ***
      END
C
C
      SUBROUTINE NUPAGE
C    DUMMY NEW PAGE ROUTINE
       PRINT 2000
 2000  FORMAT(1H1)
      RETURN
      END
C
C
      REAL*8 FUNCTION TOHMS(P1)
C     FUNCTION RETURNS P IN HH MM SS FORMAT
      IMPLICIT REAL*8(A-H,O-Z)
      DATA ONE/1.0D0/
      P3 = DABS(P1)
      B = DMOD(P3,ONE)*60.
      C = DINT(B)
      D = C/100.
      P2 = DINT(P3) + D + DMOD(B,ONE)*60./10000.
      P2 = P2*DSIGN(ONE,P1)
      TOHMS = P2
      RETURN
      END
